function  [V, T, F, mvcount, vsize, blksize] = block_lanczos(...
           V, T, F, vsize, maxvsize, blksize, blkcontrol, orthF, mvcount)
% 
%  Usage:
%       [V, T, F, mvcount] = block_lanczos(V, T, F, vsize, maxvsize, ...
%			     blksize, blkcontrol, orthF, mvcount)
%
%  Given a size (vsize) Lanczos decomposition of an operator OP:
%
%       (OP)*V(:, 1:vsize) = V(:, 1:vsize)*T + F*E'     
%
%  note that if vsize>0, then on input the condition F'*V(:, 1:vsize)=0 
%  must be satisfied.
%
%  block_lanczos computes additional several steps block Lanczos resulting in
%  a size (vfsize) Lanczos decomposition:  (vfsize<=maxvsize)
%
%          (OP)*V(:, 1: vfsize) = V(:, 1: vfsize)*T + F*E'
%  with       F'*V(:, 1:vfsize) = 0
%   
%  The block matrix-vector product W=OP*V is computed by an user provided
%  external function called User_MatVec_Mult, with interface:
%   
%            [ W ] = User_MatVec_Mult( ndim, V );  
%   
%   
%===== Input/Output variables:   
%   
%   V: (Input/Output)
%       On input the orthonormal basis from a previous size vsize block
%          Lanczos/Krylov decompisition.  
%       On output, the orthonormal basis of the size vfsize block
%          Lanczos/krylov decompisition.
%   
%   T: (Input/Output) the Rayleigh-Quotient matrix (mostly block diagonal and
%      tri-diagonal except just after contraction of the Krylov-Spectral method)
%   
%   F: (Input/Output) the residual vectors (On input, if vsize=0, F contains the
%       initial vectors)  
%
%   vsize: on input, the size of V that are already computed, 
%          on output, the size of V (input V together with the curently computed V)
%   
%   blksize: block size
%
%   blkcontrol:  ==  0,   fixed block size  (default)
%                == -1,   block size non-increasing
%                ==  1,   block size can increase (this one does not fit when not
%                         Rayleigh-Ritz step is involved. usually it is useful
%                         right after restart. inside this subroutine we increase 
%                         block size only when F becomes zero)
%
%   orthF:  ==  1,   if the input F is orthonormal  (qr(F,0) not necessary)
%           ==  0 or anything else,       if the input F is not orthonormal  
%                                         (qr(F,0) necessary)
%
%   mvcount:  integer to count the number of matrix vector multiplications
%

  DEBUG = 0;   %--1 debug, 0 not debug
  
  %
  % vsize is the current V size, (this index makes life easier, especially
  % for the blocksize-varying case).
  % vsize must be updated after each Lanczos step
  %
     
  ndim = size(F,1);
  colF = size(F,2);
  
  if (vsize>0 & (size(V,1) ~= ndim)),
    error('size V and size F mismatch');
  end  
  
  if ( ~exist('blksize','var') ), 
    blksize = colF; 
  end
  
  if ( ~exist('blkcontrol','var') ), 
    blkcontrol = 0; 
  end
  
  if ( ~exist('orthF','var') ), 
    orthF = 0; 
  end

  if ( colF < blksize),
    %
    % augment F to have blksize columns
    %
    F(1:ndim, colF+1:blksize) = rand(ndim, blksize-colF);
    colF = blksize;
    orthF = 0;
    
  elseif ( colF > blksize) 
    %
    % use only the first blksize columns of F
    %
    %F = F(1:ndim, 1:blksize) 
    %
    % or, use larger blocksize
    %
    blksize = colF;
  
  end

  if ( vsize == 0 ),
    %  
    % start the Lanczos decomposition from scratch, the input F
    % can not be viewed as residual vectors here, the first block
    % of V comes from F and we need to compute the residuals.
    %
    if ( orthF == 1 ),
      %
      %  avoid QR if F is already orthonormal
      %
      V(:,1:blksize) = F;
      rankR = colF;
    else
      [V(:,1:blksize), R] = qr(F, 0);
      %[V(:,1:blksize), R] = qr(F(:,1:blksize), 0);
      rankR = rank(R);
    end
    
    if ( rankR < colF  & rankR > 0 ), 
      %
      % the rank deficient case. this rank deficiency at the first
      % step of iteration is easy to handle.
      %
      if (blkcontrol == -1),
	blksize = rankR;
      end
    elseif (rankR == 0)
      %
      % need to reset initials
      %
      [V(:,1:blksize)] = reset_initial(ndim, V, vsize, blksize);	
    end    
    
    %
    % compute the block matrix-vector products
    %
    [ F ] = User_MatVec_Mult( ndim, V(:,1:blksize) );
    mvcount = mvcount + blksize;
    
    %
    % compute the first diagonal block of T
    %
    T(1:blksize, 1:blksize) =  V(:,1:blksize)'*F;
    
    %
    % compute the new residual vectors, and 
    % make sure F is orthogonal to V(:,1:blksize)
    %
    for j = 1 : blksize
      nrmold(j) = norm(F(:,j));
    end
    F = F - V(:,1:blksize)*T(1:blksize, 1:blksize);
    [F] = selected_reorth(F, V(:,1:blksize), nrmold(1:blksize), ...
			  T(1:blksize, 1:blksize) );
   
    %
    % update the size of current basis
    %
    vsize = blksize;
    
  end


  while (vsize <= maxvsize-blksize),
    
    colrange = [vsize+1 : vsize+blksize];

    %rankR = rank(F);
    [V(:,colrange), R] = qr(F, 0);  
    %%% no piviting can be wrong, must check rank to see if pivoting
    %%% is necessary or not.  if yes, use the following.
    %%% [V(:,colrange), R, pivt] = qr(F, 0);   
    %%% R = R(:, invert_pivt(pivt));
    
    rankR = rank(R);
    
    if (rankR < size(R,1) & rankR > 0), 
      %
      % handle the rank deficient case (mainly do the pivoted QR, a must do)
      % 
      if ( DEBUG == 1 ),
	fprintf(' **** R rank deficient, j = %i, rank(R)=%i\n', j, rankR);
      end
      
      [V(:,colrange), R, pivt] = qr(F, 0);   
      R = R(:, invert_pivt(pivt));    
      
      if (blkcontrol == -1),  

	rowrange = [vsize+1 : vsize+rankR];
	
	%
        % assign the reduced subdiagonal block
	%
	T(rowrange, vsize-blksize+1:vsize) = R(1:rankR, 1:blksize);
	
	%
	% update the shrinked block size and colrange
	%
	blksize = rankR;
	colrange = [vsize+1 : vsize+blksize]; 
	
      else
	%
        % keep the blocksize fixed, need to orthonormalize
	% the vectors in V(:, vsize+rankR+1:vsize+blksize) to 
	% V(:, 1: vsize). 
	%
	newrange =  [vsize+rankR+1:vsize+blksize];
	htmp = V(:, 1: vsize)'*V(:,newrange);
	for j = 1: blksize-rankR
	  nrmold(j) = norm(V(:,vsize+rankR+j));
	end

	V(:,newrange) = V(:,newrange) - V(:, 1: vsize)*htmp;
	
	V(:,newrange) = selected_reorth(V(:,newrange),  ...
			V(:,1:vsize),  nrmold(1:blksize-rankR));
	
	%
	% the above two lines are NOT enough for an orthonormal basis!
	% need an additional QR step, or do the normalization.
	%
	[V(:,newrange), Rtmp]=qr(V(:,newrange),0);

	if (DEBUG == 1),
	  orth_err = norm(V(:,newrange)'*V(:, 1: vsize));
	  eye_err = norm(V(:,vsize+1:vsize+blksize)'*V(:,vsize+1:vsize+blksize) ...
			 - eye(blksize));
	  fprintf('rank deficient: orth_err = %e,  eye_err = %e\n', orth_err, eye_err) 
	end	

	%
        % assign the subdiagonal block
	% %%% seems it should be better to zeros out R(:,rankR+1:blksize)
	% %%% if we stick to use block tridiagonal? (they are already 0s)
	%
	T(vsize+1:vsize+rankR, vsize-blksize+1:vsize) = R(1:rankR, 1:blksize);
	%T(vsize+rankR+1:vsize+blksize, vsize-blksize+1:vsize) = ...
	%                                         zeros(blksize-rankR, blksize);
	
      end    
      
    elseif (rankR == 0)
	%
	% invariant subspace has been found, need to restart with completely
	% new vectors that are orthogonal to the existing Lanczos basis.
	%
	[V(:, vsize+1:vsize+blksize)]= ...
	                    reset_initial(ndim, V(:,1:vsize), vsize, blksize);
	if (DEBUG == 1),
	  fprintf('vsize = %i,  rankR = %i,  blksize=%i\n', vsize, rankR, blksize) 
	  newrange =  [vsize+1:vsize+blksize];
	  orth_err = norm(V(:,newrange)'*V(:, 1: vsize))
	end
	
	% 
	% the subdiagonal block of T need not be updated here since it is all zeros.
	%
	
    else  % R full rank, update the subdiagonal block of T as R

      T(colrange, vsize-blksize+1:vsize) = R;

      if (DEBUG == 1),
	newrange =  [vsize+1:vsize+blksize];
	orth_err = norm(V(:,newrange)'*V(:, 1: vsize));
	eye_err = norm(V(:,vsize+1:vsize+blksize)'*V(:,vsize+1:vsize+blksize) ...
		     - eye(blksize));
	if ( orth_err > 1e-10  | eye_err > 1e-10),
	  fprintf('vsize = %i,  rankR = %i,  blksize=%i\n', vsize, rankR, blksize)  
	  fprintf('orth_err = %e,    eye_err = %e\n',  orth_err, eye_err) 
	end
      end
	              
    end
    
    %
    % update the current basis size, necessary for the next iteration.
    %
    vsize = vsize + blksize;
	
    %
    % compute the user block matrix-vector products, using F as temp storage
    %
    [ F ] = User_MatVec_Mult( ndim, V(:,colrange) );
    mvcount = mvcount + blksize;
    
    %
    % compute original F norm for the purpose of selected reorthogonalization
    %
    for j = 1:blksize
      nrmold(j) = norm(F(:,j));    
    end
                   
    %
    % compute the new columns of T, here we stick to a block tri-diagonal T.
    % we use the upper triangular part of T as temp storage.
    %
    T(1:vsize, vsize-blksize+1:vsize) = V(:,1:vsize)'*F;
    
    %
    % full reorthogonalization, (W against V(:,1:vsize))
    %
    F = F - V(:,1:vsize)* T(1:vsize, vsize-blksize+1:vsize);
      
    %
    % make sure F is orthogonal to V(:,1:vsize). 
    %
    % update the blksize columns of the current last blksize columns of T,
    % this includes the update of the recently computed diagonal block

    [F] = selected_reorth(F, V(:,1:vsize), nrmold(1:blksize), ...
			  T(1:vsize, vsize-blksize+1:vsize));

    if (DEBUG == 1), 
       orth_err = norm(V(:,1:vsize)'*V(:,1:vsize) -eye(vsize));
       final_F_orth_V = norm(F'*V(:,1:vsize));
       if (  final_F_orth_V  > 1e-10),
	 fprintf('  final_F_orth_V = % e \n',  final_F_orth_V)
       end
       if (  orth_err  > 1e-10),
	 fprintf('  final_orth_V_err = % e \n',  orth_err)
       end      
    end
    
  end
  
        
  %
  % finally, symmetrize T, note we only use the lower triangular part of T
  %
  %%% T = (T+T')/2;
  T = tril(T)+tril(T)'-diag(diag(T));
 
  vsize=vsize;
 % sizeT = size(T)

 