function [eigval, V, nconv, T, info, restart, mvcount]= ks(varargin);
%   
%  Usage: 
%   [eigval, V, nconv, T, info] = ks(nneed, F, opts);
%    
%  or
%   [eigval, V, nconv, T, info] = ks(nneed, opts);
%   
%   
%  Krylov-Spetral implements a block version Krylov-Spectral method to compute   
%  the smallest nneed eigen-pairs of a symmetric matrix (A). The Matrix is implict, 
%  users need to provide a function with interface: 
%                    [ W ] = User_MatVec_Mult( ndim, V, A );   
%  for the matrix-vectors products W = (A)*V. 
%   
%
%===== Input variables
%
%  nneed  --- number of needed eigen-pairs
%  F      --- initial vectors (optional, if omitted, the matrix is
%             default to a global matrix A, and F contains random vectors)
% 
%
%  ===== optional parameters in the opts: ===========================
%
%  matrix --- the matrix/operator 
%             (if omitted, it is assumed to be a global matrix with
%             name A) 
%
%  nkeep  --- how many columns to keep after basis contraction
% 
%  maxvsize--- largest subspace dimention for the block Lanczos.
% 
%  tol    --- convergence tolerance. Convergence is determined when             
%             || (OP)*x - lambda*x ||_2 <= tol*||(OP)||_2. where ||(OP)||_2 is 
%             approximated by the largest absolute Ritz value.  
%             the default tol is: 2.2205D-16
%
%  blksize --- block size 
% 
%  blkcontrol --- integer to control how to adjust the block size
%                 ==  0,   fixed block size  (default)
%                 == -1,   block size can decrease
%                 ==  1,   block size can increase (especially after contraction
%                          when it is detected that larger than the current blksize
%                          cluster exists)
% 
%  orthF  --- iteger used to signal if the initial vectors F is orthonormal
%             == 1,    F is orthonormal  (qr(F,0) not necessary)
%             == anything else,   F is not orthonormal (qr(F,0) necessary)
% 
%  max_restart --- maximum restarts allowed
% 
% 
%===== Output variables 
%
%  eigval --- the converged eigenvalues 
% 
%  nconv  --- number of converged eigen-pairs
%
%  V      --- the Ritz vectors, V(:,1:nconv) contains the converged eigenvectors.
%  
%  info   --- = 0  (if converged),   (= number of restart) if not convergent
%
%  The output V and T satisfy the invariant subsapce relation:
%
%        (OP)*V(:,1:nconv) = V(:,1:nconv)*T(1:nconv,1:nconv)
%
%  the outputs in V(:,nconv+1:m*blksize) are optional
%  
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Krylov-Spectral Algorithm:  (b denotes block size: blksize)
%   
%   1.  m-step block_lanczos decomposition:
%        (OP) * V(:,1:mb) = V(:,1:mb) * T(1:mb,1:mb) + F(:,1:b)* E'
%   
%   2.  spectral decomposition of T_mb:
%         T(1:mb,1:mb) * Q(:,1:mb) = Q(:,1:mb) * D(1:mb,1:mb)
%   
%   3.  keep only the first k block of D and the corresponding eigenvectors of T:
%       (D(:,1:nkeep), Q(:,1:nkeep))
%   
%   4.  contract the basis:
%         V(:,1:nkeep) = V(:,1:mb) * Q(:,1:nkeep)
%   
%   5.  do thin QR decomposition of F(:,1:b):  
%         [U, R] = qr(F(:,1:b), 0)
%   
%   6.  determine deflation from R*E'*Q(:,1:nkeep).
%       use nconv as convergence index, i.e., V(:,1:nconv) are converged Ritz vectors.  
%       if after permutation, R*E'*Q_bk = [zeros, B], then nconv=nconv+size(zeros,2),
%       if nconv >= nneed, exit.  
%
%   7.  update T_nkeep:  (need to consider the deflated case)
%          T(1:(k+1)b, 1:nkeep) = [  D(1:nkeep,1:nkeep) ] = [  D(1:nkeep,1:nkeep) ]
%                                 [  R*E'*Q(:,1:nkeep) ]    [  R*Q((m-1)b+1:mb,1:nkeep) ]
%   
%   8.  augment the Lanczos basis: 
%          V(:,1:(k+1)b) = [V(:,1:nkeep), U]
%   
%   9.  compute user-matrix-vectors multiplications:
%          W = (OP) * U
%  
%  10.  compute the residual vectors that are orthogonal to V(:,1:(k+1)b):
%          htmp = V(:,1:(k+1)b)'*W
%          F = W - V(:,1:(k+1)b)*htmp   
%       (do re-orthogonalization if necessary to make sure F is orth to V)
%   
%  11.  compute the (k+1)-th diagonal block of T as the last b block of htmp:
%          T(nkeep:(k+1)b, nkeep:(k+1)b) = htmp(nkeep:(k+1)b, 1:b)
%   
%  12.  call block_lanczos to do additional m-k-1 step Lanczos decomposition
%       to get V_mb and T_mb  (note that in the deflated case, F need to be
%       orthogonal also to the defalted Ritz vectors);  goto 2.  
%      
%   

    global A

  %
  % list all the rest default values, they will be overwritten 
  % if provided through structure opts
  %
  tol         = 1e-10;
  blksize     = 3;
  DEBUG       = 0;      %--1 debug, 0 not debug
  
  if nargin == 0, help ks, return, end
  
  nneed = varargin{1};
  
  if (nargin < 2), 
      ndim = size(A,1);
      F = rand(ndim, blksize);
  else
      if isa(varargin{2},'struct'),
          opts = varargin{2};
          F = rand(ndim, blksize);
      else
          F = varargin{2};
      end      
  end
  
  [ndim,colF] = size(F);
  
  max_restart = max(floor(ndim/2), 300);
  blkcontrol  = 0;
  orthF       = 0;      %default: F needs to be orthogonalized 
  Anumeric    = 'yes';  %the defalut matrix is numeric
  maxvsize    = max(2*blksize*nneed, 20);
  nkeep       = floor(maxvsize/2); 

  
  if (nargin > 2)
    opts = varargin{3};
    if ~isa(opts,'struct')
        error('Options must be a structure.')
    end
  end
      
  if (nargin > 2 | isa(varargin{2},'struct')), 
    if isfield(opts,'matrix'),
        A=opts.matrix;
        if ~isnumeric(A),
            Anumeric='no'; 
        end
    end    
    
    if isfield(opts,'tol'),
        tol=opts.tol;
    end

    if isfield(opts,'blksize'),
        blksize=opts.blksize;
    end

    if isfield(opts,'nkeep'),
        % better to adaptively choose nkeep (to be added later)
        nkeep=opts.nkeep;
        if (nkeep < nneed/blksize), error ('nkeep too small'), end
    end
    
    if isfield(opts,'maxvsize'),
        maxvsize  = max(maxvsize, opts.maxvsize);
        nkeep     = floor(maxvsize/2); 
    end
    
    if isfield(opts,'max_restart'),
        max_restart=min(opts.max_restart, max_restart);
    end    
    
    if isfield(opts,'blkcontrol'),
        blkcontrol=opts.blkcontrol;
    end   
    
    if isfield(opts,'orthF'),
        orthF=opts.orthF;
    end       
  
    if isfield(opts,'DEBUG'),
        DEBUG=opts.DEBUG;
    end     
  
    if isfield(opts,'a_eig_exact'),
        % used only for debugging 
        a_eig_exact =opts.a_eig_exact;
    end     
  
  end
           
  if ( maxvsize > ndim), error('maxvsize should be < ndim'); end
  if (DEBUG~=0 & Anumeric=='yes'),
      if(~isequal(A, A')),
          error('your input matrix is not symmetric/Hermitian');
      end
  end
  
  %
  % pre-assign storage for efficiency, matrix from now on 
  % should be accessed by explicit indexes
  %
  T = zeros(maxvsize, maxvsize);
  V = zeros(ndim, maxvsize);
  eigval = zeros(nkeep,1);
  

  %
  % initial block Lanczos to V os size maxvsize
  %
  mvcount =0;
  [V, T, F, mvcount, vsize, blksize] = ...
      block_lanczos([], [], F, 0, maxvsize, blksize, blkcontrol, orthF, mvcount);
  
  if (DEBUG == 1),
    ortherr1 = norm(V'*V - eye(size(V,2)));
    ortherr2 = norm(F'*V(:,1:size(V,2)));
    if (ortherr1>1e-10 | ortherr2>1e-10),
      fprintf('--- after block_lanczos_zero,  orth_err = %e\n', ortherr1);
      fprintf('--- after block_lanczos_zero,  F_orth_V = %e\n', ortherr2);
      figure
      spy(T)
    end   
  end
  
    
  %
  % begin the main iteration
  %
  nconv = 0;
  restart = 0;
  
  while (restart <= max_restart),

    nconv1 = nconv+1; 

    %
    % compute the spectral decomposition of T(nconv+1:mb,nconv+1:mb)
    %
    %mb = size(T,1);
    mb=vsize;
    [Q, T(nconv1:mb,nconv1:mb)] = eig(T(nconv1:mb,nconv1:mb));
  
    %
    % update the first nkeep columns of non-convengent Ritz vectors in V.
    %
    % the largest dimension for the Ritz vectors (that other new vectors
    % need to orthogonalize against) is nconv+nkeep, since nconv may change,
    % we use nconvkb to store the current nconv+nkeep (for this iteration
    % step,  nconvkb should remain fixed even though nconv may increase).
    %
    nconvkb = nconv + nkeep;
    V(:,nconv1:nconvkb) = V(:,nconv1:mb)*Q(1:mb-nconv1+1,1:nkeep);
    
    if (DEBUG == 1  &  ~isempty(a_eig_exact) ),
      %a_eig_exact(1:nneed) - diag(T(1:nneed,1:nneed))
      eig_diff= a_eig_exact(1:nconvkb)-diag(T(1:nconvkb,1:nconvkb))
    end
     
    %
    % store the new basis (the Q factor of the QR factorization of F)
    % in V(:,nconvkb+1:nconvkb+blksize).
    % note it is important not to use nconv as index since nconv may
    % change before the data stored in this temporaty storage is used again.
    %
    blksize = size(F,2);
    [V(:,nconvkb+1:nconvkb+blksize), R] = qr(F,0);
    rankR=rank(R);
    if ( rankR < blksize),
      fprintf('\n  ******** blksize=%i,  rank(R)=%i\n', blksize, rankR);
      if (blkcontrol == -1),  
	%
        % assign the reduced subdiagonal block
	% **** not right here, you still need pivoted QR !!
	%
	sizeR=size(R(1:rankR,:))
	sizeQ=size(Q(mb-nconv-blksize+1:mb-nconv, 1:nkeep))
	ResidualV(1:rankR,1:nkeep) = R(1:rankR,:)*Q(mb-nconv-blksize+1:mb-nconv, 1:nkeep);
	%
	% update the shrinked block size 
	%	
	blksize = rankR
	nconvkbb = nconvkb+blksize;
      else
	%
	% need to make V(:,nconvkb+rankR+1:nconvkb+blksize) orthogonal to
	%  V(:,1:nconvkb+rankR)
	%
	[V(:,nconvkb+1:nconvkb+blksize), R, pivt] = qr(F,0);
	R = R(:, invert_pivt(pivt)); 
      
	newrange = [nconvkb+rankR+1:nconvkb+blksize];
	htmp =  V(:,1:nconvkb+rankR)'*V(:,newrange);
	V(:,newrange)=  V(:, newrange) - V(:,1:nconvkb+rankR)*htmp;
	for j = nconvkb+rankR+1:nconvkb+blksize
	  nrmold(j-nconvkb-rankR) = norm(V(:,j));
	end
	V(:,newrange)=  selected_reorth( V(:,newrange), V(:,1:nconvkb+rankR), ...
					 nrmold(1:blksize-rankR));
	[V(:,newrange), Rtmp]=qr(V(:,newrange),0);
		 
	ResidualV(1:blksize,1:nkeep) = ...
	  [R(1:rankR,:)*Q(mb-nconv-blksize+1:mb-nconv, 1:nkeep); zeros(blksize-rankR, nkeep)];
	nconvkbb = nconvkb+blksize;

      end
      if (DEBUG == 1)	
	Orthogonal_err = norm(V(:,1:nconvkbb)'*V(:,1:nconvkbb) - eye(nconvkbb))    
      end
    
    else    
      %
      % full rank case:
      % directly update the residual (R * the last blksize rows of Q(:,1:nkeep)) 
      %
      %% ResidualV(1:blksize, 1:nkeep)=R*Q((k-1)*blksize+1:nkeep, 1:nkeep); %%WRONG!!
      ResidualV(1:blksize,1:nkeep) = R*Q(mb-nconv-blksize+1:mb-nconv, 1:nkeep);
      
    end
    
    %
    % determine convergence and deflate converged Ritz pairs.
    % the converged Ritz vectors will be stored at the beginning nconv 
    % columns of V, ie. V(:, 1:nconv)
    %
    %loc_tol = tol * max([abs(T(mb,mb)),abs(T(1,1)),1]);
    loc_tol = tol * max([abs(T(mb,mb)),abs(T(nconv1,nconv1)),1]);
    
    % count the local number of convergence
    conv1 =0;  
    
    for j = nconv1 : nconvkb

      residual_nrm = norm(ResidualV(:,j-nconv1+1));
      if ( residual_nrm > loc_tol),
          % exit the loop when non-convergence is encountered
          break;
      else
	
	%
	% one ritz pair convered, bookmark in eigval, note down the computed
	% residual norm
	%
	conv1 = conv1+1;
	nconv = nconv+1;
	eigval(nconv) = T(j,j);   %%still a bit buggy here, why??
	                    %% now fixed after adding "break" above
                            %% reason: must make sure nconv == j
		            %% since you no longer do swap for the nconv < j case
	residual_norm(nconv) = 	residual_nrm;	 
			 
	if (DEBUG == 1),
	  tmp_nrm = compute_res(ndim, V(:,nconv), eigval(nconv));
	  fprintf('nconv=%i,  ||ResidV(:,%i)||= %e', nconv,j-nconv1+1,...
		  residual_nrm );
	  fprintf(',  actual ||residv||= %e\n', tmp_nrm)
	end

	if ( j > nconv ),	
	  
	  break;   %% exit the loop after first non-convergence!!
	           %% the following swap does work and has no bug,
		   %% but it can not guarantee that the converged eigenvalues
		   %% are among the smallest nneed, especially if the swap
		   %% happens at the end of convergence (some larger 
		   %% eigenvalues can converge faster than the smallers ones
		   %% and the larger eigenvalues are returned (this is not
		   %% desirable)
	  
	  %
	  % swap V(:,j) with V(:,nconv)--currently not convergent if j>nconv.
	  % use one free column in V as temp storage
	  %
	  fprintf(' *****swap %i with %i,  resnrm(V(:,%i))=%e\n', j, ...
		  nconv,j,norm(ResidualV(:,j-nconv1+1))  );
	  V(:, nconvkb +blksize+1) = V(:, j);
	  V(:, j) = V(:, nconv);
	  V(:,nconv) = V(:, nconvkb +blksize+1);
	  
	  %
	  % swap the converged Ritz value to the beginning of T. (since 
	  % converged Ritz values are stored in eigval, it is possible to save 
	  % only the unconverged Ritz value).
	  %
	  tmp = T(j,j);
	  T(j,j)=T(nconv,nconv);
	  T(nconv,nconv)=tmp;

	  %
	  % only save the non-zero ResidualV. the beginning nconv columns of 
	  % ResidualV correspond to converged Ritz pairs, no need to save them.
	  % 
	  ResidualV(:,j-nconv1+1)=ResidualV(:,nconv-nconv1+1);
	
	  if (DEBUG == 1)	
	    fprintf(' *****resnrm(V(:,%i))=%e\n', nconv,  ...
		    norm(ResidualV(:,nconv-nconv1+1))  );
	  end
	end
      end
    
      %
      % the converged Ritz pairs are in (T(j,j), V(:,j)),  j=1:nconv;
      % note converged Ritz value are also stored in eigval (eigval(j) = T(j,j)).
      %
      if (nconv >= nneed), 
	info = 0;

        eigval = eigval(1:nconv);
        V = V(:, 1:nconv);
        
	fprintf('\n krylov-spectral computed residual norms \n');
	fprintf(' %11.5e   %11.5e   %11.5e   %11.5e   %11.5e\n', residual_norm(1:nconv))
	[ernrm_max_ks, idx] = max(residual_norm(1:nconv));
	fprintf(' max error norm:  errnrm( %i ) = %e\n\n', idx, ernrm_max_ks);
	
	fprintf('----------------------------------------------- \n' );
	fprintf(' number of restarts = %i,   Mv counts= %i\n', restart, mvcount);
	fprintf(' nconv= %i,   nneed= %i,    blksize= %i\n', nconv, nneed, blksize)
	fprintf(' Krylov-Spectral successfully converged! \n' );	
	fprintf('-----------------------------------------------\n\n' );	

	return; 
      end
    
    end


    %
    % update T, we opt to keep all T(1:nvonv+nkeep,1:nconv+nkeep) instead of
    % deflating the T(1:nconv, 1:nconv)
    %
    nconvkb_b = nconvkb+blksize;
    T(nconvkb+1:nconvkb_b, nconv+1:nconvkb) =  ResidualV(1:blksize,conv1+1:nkeep);  
    T(nconv+1:nconvkb, nconvkb+1:nconvkb_b) =  ResidualV(1:blksize,conv1+1:nkeep)'; 

    if (DEBUG == 1)	
      for kkkk = 1: blksize
	nrm_1 = norm(ResidualV(kkkk,conv1+1:nkeep));
	fprintf('norm(ResidualV(%i,  %i : %i))=%e\n', kkkk,conv1+1,nkeep, nrm_1 )
      end
      %figure; spy(T); pause
    end
      
    %
    % compute User_Matrix_vectors products.  use the next free blksize columns
    % of V as temp storage
    %
    if (Anumeric=='yes'),
        [V(:,nconvkb_b+1:nconvkb_b+blksize)]= ...
                      User_MatVec_Mult(ndim, V(:,nconvkb+1:nconvkb_b));
    else
        [V(:,nconvkb_b+1:nconvkb_b+blksize)]= ...
                      User_MatVec_Mult(ndim, V(:,nconvkb+1:nconvkb_b),A);        
    end
    mvcount = mvcount + blksize;
    
    
    %%
    %% compute new residual vectors F, orthogonalize F against V(:,1:nconvkb+blksize)
    %%
    htmp = V(:,1:nconvkb_b)'*V(:,nconvkb_b+1:nconvkb_b+blksize);
    %
    % compute the norm for the purpose of telling if 2nd step Gram-Schmidt is necessary
    %
    for j = nconvkb_b+1:nconvkb_b+blksize
      nrmold(j-nconvkb_b) = norm(V(:,j));
    end
    %
    % first step block Gram-Schmidt
    %
    F = V(:,nconvkb_b+1:nconvkb_b+blksize) -  V(:,1:nconvkb_b)*htmp;
    %
    % selected second step Gram-Schmidt reorthogonalization to make sure
    % F is orthogonal to V(:,1:nconvkb_b)
    %
    [F] = selected_reorth(F, V(:,1:nconvkb_b), nrmold(1:blksize), ...
    			  htmp(1:nconvkb_b, :));
    
    %
    % compute the new diagonal block of T, it is the last block of htmp
    %
    % that is: T(kbnew+1:kbnew+blksize,kbnew+1:kbnew+blksize) = 
    %                                 htmp(nconv+kb+1:nconv+nkeep+blksize,1:blksize);
    % where kbnew = nconvkb = nconv+nkeep
    % note that the current size of T is  nconvkb_b = nconvkb + blksize.
    % 
    T(nconvkb+1:nconvkb_b, nconvkb+1:nconvkb_b) = ...
                                     htmp(nconvkb+1:nconvkb_b,1:blksize);
    
    
    %% 
    %% do additional steps block Lanczos decomposition
    %% 
    orthF=0;
  
    if (DEBUG == 1),
      ortherr1 = norm(V(:,1:nconvkb_b)'*V(:,1:nconvkb_b) - eye(nconvkb_b));
      ortherr2 = norm(F'*V(:,1:nconvkb_b));
      if (ortherr1>1e-10 | ortherr2>1e-10),
	fprintf('--- before block_lanczos,  orth_err = %e\n', ortherr1);
	%pause	fprintf('--- before block_lanczos,  F_orth_V = %e\n', ortherr2);
        %T(nconvkb_b+1:maxvsize,nconvkb_b+1:maxvsize)=zeros(maxvsize-nconvkb_b); 
	figure
	spy(T)
	fprintf(' current vsize=%i,  final vsize = %i,  size(V) = %i %i \n', ...
                nconvkb_b, maxvsize, size(V))
      end
    end
    maxvsize = maxvsize +conv1;
    [V, T, F, mvcount, vsize, blksize]= block_lanczos(V,  T,  F,  nconvkb_b,  ...
                         maxvsize, blksize, blkcontrol, orthF, mvcount);
    if (DEBUG == 1),   
      ortherr1 = norm(V(:,1:nconvkb_b)'*V(:,1:nconvkb_b) - eye(nconvkb_b));
      sizeV = size(V);
      %ortherr = norm(V'*V - eye(size(V,2)));
      if (  ortherr1 > 1e-10 ),
	fprintf('***  after block_lanczos,  orth_err = %e\n', ortherr1);
	figure
	spy(T)
	fprintf('     size(V) = %i %i \n', size(V))
      end
    end
     
    restart = restart +1;
    
    if (restart > max_restart),
      info = restart;
      fprintf('**** krylov-spetral does NOT converge in %i restarts\n', max_restart);
    end
    
    
  end
    
