function  [A]=model(id, n1, n2, n3)
%
% Usage:  [A]=model(id);  
%
% Reads a .mtx file corresponding to the input id to create a matrix.
% The matrix is saved in a gloabl matrix named A if no output argument
% is provided.
%
% The file  "mmread.m"  from the matrix market is needed for
% processing data from the .mtx file(s).
%
% Note that absolute paths for the data files are not provided.
% users should make sure that the paths for their .mtx files are in the 
% MATLAB path. This can be easily done by the "addpath" command, 
% or by setting the correct path(s) in the "startup.m" file.
%
% For unix/linux system, it is enough to set an env variable named MATLABPATH
% in your .cshrc or .bashrc, and put the startup.m file in the MATLABPATH dir.
% Then every time matlab starts up, it will source this startup.m to set your
% preferred initial settings. (that is, you don't have to put this startup.m
% in every dir in which you start up matlab)
%
%---------------------------------------------------------------------------
%
%==== the following are id numbers relating to often used models:
%  (using an integer id instead of a string name is not good for human's 
%   memory, but good for the "computer loop". the following output is 
%   provided to remember some id's more easily)
%
%  id for Si(m)H(n) models:
%    10 -- Si-2 (smaller); 11 -- Si;    12 -- SiH4;      13 --  Si10H16;   
%    14 -- Si34H36;        15 -- CO;    16 -- SiO;       17 --  SiO2; 
%    18 -- H2O;            19 -- Na5;   20 -- benzene;   21 --  SiNa;
%    22 -- si5h12;      23 -- GaAsH6;   24 -- Ga3As3H12; 25 --  Ga10As10H30;
%    26 -- Ga19As19H42; 27 -- Ge87H76;  28 -- Ge99H100;  29 --  Ga41As41H72;
%    30 -- Si87H76
%
%==== if no .mtx files are available, some Laplacians can be generated:
%  Usage (for id number ranged in [41:49])
%
%    model(id, n1,n2,n3);   where the dimension n1, n2, n3 are optional.
%
%  id number corresponds to:
%    41-- 1d-laplacian;     42-- 2d-laplacian;     43-- 3d-laplacian;  
%    44-- 2d-lap (L-shape); 45-- 2d-lap (H-shape); 46-- 2d-lap (A-shape);
%    47-- 2d-lap (Nested);  48-- 2d-lap (Disc);    49-- 2d-lap (C-shape)
%
%  example:
%    A=model(43, 10, 20, 30); generates a 3-d laplacian with dim 10x20x30. 
%
%    A=model(49, 100); generates C-shape 2-d laplacian,
%      it is essentially:  delsq(numgrid('C', 100));
%      (i found "model(id ...)" more handy than "delsq(numgrid(shape ...))")
%
%==== typing "model" or "help model" prints out the above.
%
  
%
%--y.k. zhou (zhou@msi.umn.edu)
%  UMN, 2005
%

  if (nargin == 0), help model; return; end
  global n  x1   
  global mode  Title  filename  
  global Matsymm  %Matsymm=1 if symmetric, Matsymm=0 if not symmetric/Hermitian
  global opert    %order of perturbation, often not necessary

  % note that syntax [A]=model(id) is no good, it should be model [A]=model(id);.
  % one should never forget the ; in calling model(id), otherwise one gets too
  % many unwanted screen output if A is large.)  
  if (nargout == 0),
    global A
  end
  
  switch id
    
   case 1
     [ A ] = mmread('olm500.mtx');
     %% very non-symm
     n = size(A,1);    Matsymm=0;
     x1 = ones(n,1);   
     mode = 'LR';   %pretty hard
     %mode = 'SR';    
     fprintf('  \n==========   olm500.mtx,  x1=ones(n,1),    n = %i\n', n)     
     fprintf(' real unsymmetric,  seeks the rightmost eigenvalues\n')
     Title = [' olm500,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='olm500';
     opert = 1e-5;  

   case 2
     [ A ] = mmread('olm1000.mtx');
     %% very non-symm
     n = size(A,1);     Matsymm=0;
     x1 = ones(n,1);    mode = 'LR';
     %mode = 'LR';   %pretty hard
     mode = 'SR';   
     fprintf('  \n==========   olm1000.mtx,  x1=ones(n,1),    n = %i\n', n)     
     fprintf(' real unsymmetric,  seeks the rightmost eigenvalues\n')
     Title = [' olm1000,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='olm1000';
     opert = 1e-5;
     
   case 3
     [ A ] = mmread('pde900.mtx');
     %% non-symm
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'LR';
     fprintf('  \n==========   pde900.mtx,  x1=ones(n,1),    n = %i\n', n)     
     fprintf(' real unsymmetric,  seeks the rightmost eigenvalues\n')
     Title = [' pde900,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='pde900';
     opert = 1e-3;

   case 4
     [ A ] = mmread('ck656.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode='LM';
     fprintf('  \n==========   ck656.mtx,  x1=ones(n,1),    n = %i\n', n)     
     Title = [' ck656,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='ck656';
     opert = 2e-1;

   case 5
     [ A ] = mmread('rdb800l.mtx');
     n = size(A,1);       Matsymm=0;
     x1 = ones(n,1);      mode = 'LR';
     fprintf('  \n==========   rdb800l.mtx,  x1=ones(n,1),    n = %i\n', n)     
     fprintf(' real unsymmetric, Reaction-diffusion Brusselator Model \n')
     Title = [' rdb800l,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='rdb800l';
     opert = 1e-3;
     
   case 6
     [ A ] = mmread('rdb968.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'LR';
     fprintf('  \n==========   rdb968.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real unsymmetric, Reaction-diffusion Brusselator Model \n')
     Title = [' rdb968,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='rdb968';
     opert = 1e-2;

   case 7
     [ A ] = mmread('rw496.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'LM';
     %% for this model,  mode='SM' does not give correct result !??  
     %% cond(A) = 10^10
     fprintf('  \n===========   rw496.mtx,  x1=ones(n,1),    n = %i\n', n)     
     fprintf(' real unsymmetric,  Markov Chain Transition Matrix \n')
     Title = [' rw496,   real unsymmetric,   mode =', mode, ', n =', num2str(n)]
     filename ='rw496';
     opert=1e-2;


   case 8
     [ A ] = mmread('sherman4.mtx');   %n =1104
     fprintf('sheman4 \n');
     n = size(A,1);       Matsymm=0;
     x1 = ones(n,1);      mode ='LR';
     Title = ['sherman4,  real non-symmetric,  mode =', mode, ',  n =', num2str(n)]
     filename ='sherman4';
     opert =2e-2;
     
     
   case 9
     [ A ] = mmread('sherman3.mtx');    %n =5005  ---ill conditioned
     fprintf('sheman3 \n');
     n = size(A,1);   Matsymm=0; 
     x1 = ones(n,1);  mode ='LR';
     Title = ['sherman3,  real non-symmetric,  mode =', mode, ',  n =', num2str(n)]
     filename ='sherman3';    
     opert =5e-1;   

   %%  
   %-- id=10's:  symmetric models from material science  
   %%
   case 10.5
     [ A ] = mmread('si2.mtx');  %a tiny test problem, n=769
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   si2.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, si2 Model \n')
     Title = [' si2, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='si2';
     opert = 1e-1;      

   case 10
     [ A ] = mmread('Si_2sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Si_2sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, Si Model \n')
     Title = [' Si_{1}-2, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Si-2';
     opert = 1e-1;       
          
   case 11
     [ A ] = mmread('Si_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Si_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, Si Model \n')
     Title = [' Si_{1}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Si';
     opert = 1e-1;  
     
   case 12
     [ A ] = mmread('SiH4_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   SiH4_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, SiH4 Model \n')
     Title = [' Si_{1}H_{4}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='SiH4';
     opert = 1e-1;       
     
   case 13
     [ A ] = mmread('Si10H16_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Si10H16_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, Si10H16 Model \n')
     Title = [' Si_{10}H_{16}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Si10H16';
     opert = 1e-1;

   case 14
     [ A ] = mmread('Si34H36_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Si34H36_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, Si34H36 Model \n')
     Title = [' Si_{34}H_{36}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Si34H36';
     opert = 1e-1;   

   case 15
     [ A ] = mmread('CO_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   CO_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, CO Model \n')
     Title = [' C_{1}O_{1}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='CO';
     opert = 1e-1;    

   case 16
     [ A ] = mmread('SiO_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   SiO_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, SiO Model \n')
     Title = [' Si_{1}O_{1}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='SiO';
     opert = 1e-1;         
     
   case 17
     [ A ] = mmread('SiO2_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   SiO2_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, SiO2 Model \n')
     Title = [' Si_{1}O_{2}, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='SiO2';
     opert = 1e-1;         

   case 18
     [ A ] = mmread('water_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   H2O_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, H2O Model \n')
     Title = [' H2O, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='H2O';
     opert = 1e-1;     
     
   case 19
     [ A ] = mmread('Na5_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Na5_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, Na5 Model \n')
     Title = [' Na5, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Na5';
     opert = 1e-1;     

   case 20
     [ A ] = mmread('benzene_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   benzene_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, benzene Model \n')
     Title = [' benzene, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='benzene';
     opert = 1e-1;  
     
   case 21
     [ A ] = mmread('SiNa_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   SiNa_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, SiNa Model \n')
     Title = [' SiNa, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='SiNa';
     opert = 1e-1;      
     
   case 22
     [ A ] = mmread('si5h12_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   si5h12_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, si5h12 Model \n')
     Title = [' si5h12, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='si5h12';
     opert = 1e-1;     

   case 23
     [ A ] = mmread('GaAsH6_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   GaAsH6_sym.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, GaAsH6 Model \n')
     Title = [' GaAsH6, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='GaAsH6';
     opert = 1e-1;     
     
   case 24
     [ A ] = mmread('Ga3As3H12_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Ga3As3H12_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Ga3As3H12 Model \n')
     Title = [' Ga3As3H12, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ga3As3H12';
     opert = 1e-1;   

   case 25
     [ A ] = mmread('Ga10As10H30_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Ga10As10H30_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Ga10As10H30 Model \n')
     Title = [' Ga10As10H30, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ga10As10H30';
     opert = 1e-1;   
   
   case 26
     [ A ] = mmread('Ga19As19H42_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Ga19As19H42_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Ga19As19H42 Model \n')
     Title = [' Ga19As19H42, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ga19As19H42';
     opert = 1e-1;   

   case 27
     [ A ] = mmread('Ge87H76_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Ge87H76_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Ge87H76 Model \n')
     Title = [' Ge87H76, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ge87H76';
     opert = 1e-1;  
     
   case 28
     [ A ] = mmread('Ge99H100_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Ge99H100_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Ge99H100 Model \n')
     Title = [' Ge99H100, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ge99H100';
     opert = 1e-1;  

   case 29
     [ A ] = mmread('Ga41As41H72_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Ga41As41H72_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Ga41As41H72 Model \n')
     Title = [' Ga41As41H72, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ga41As41H72';
     opert = 1e-1;       
     
   case 30
     [ A ] = mmread('Si87H76_sym.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Si87H76_sym.mtx,  x1=ones(n,1),  n = %i\n', n)    
     fprintf(' real symmetric, Si87H76 Model \n')
     Title = [' Si87H76, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Si87H76';
     opert = 1e-1;     
     
     
   %%  
   %-- id=31+.    bcsstk models from NIST Matrix market
   %%
   case 31
     [ A ] = mmread('bcsstk31.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   bcsstk31.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symm indef, bcsstk31 Model \n')
     Title = [' bcsstk31, real symm. indef., mode =', mode, ', n =', num2str(n)]
     filename ='bcsstk31';
     opert = 1e-2;  
     
   case 32
     [ A ] = mmread('bcsstk32.mtx');
     n = size(A,1);
     x1 = ones(n,1);     mode = 'LR';
     fprintf('  \n==========   bcsstk32.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real Symm indef, bcsstk32 Model \n')
     Title = [' bcsstk32, real symm. indef., mode =', mode, ', n =', num2str(n)]
     filename ='bcsstk32';
     opert = 1e-2;       
     
   case 33
     [ A ] = mmread('bcsstk33.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   bcsstk33.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symm indef, bcsstk33 Model \n')
     Title = [' bcsstk33, real symm. indef., mode =', mode, ', n =', num2str(n)]
     filename ='bcsstk33';
     opert = 1e-2;  
     
       
   %%-- id=40's.    Laplacian models   
   % Note that Laplacian 1d for SR mode is a lot harder than 2d and 3d.
   %
   % It may be easier to call conv_diff or delsq(numgrid('str', n)) directly.
   % The following codes only add some necessary strings for convenience.
   %%
   case 41   %1d Laplacian
     if (nargin > 1),
       [ A ] = conv_diff(n1);
     else
       [ A ] = conv_diff(1000);
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Laplacian 1d,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 1d, mode =', mode, ', n =', num2str(n)]
     filename ='lapla1d';
     opert = 1e-2;  
     
   case 42   %2d Laplacian
     if (nargin > 2)
       [ A ] = conv_diff(n1, n2);
     elseif (nargin >1)
       [ A ] = conv_diff(n1, n1);
     else
       [ A ] = conv_diff(80, 90);
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Laplacian 2d,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d, mode =', mode, ', n =', num2str(n)]
     filename ='lapla2d';
     opert = 1e-2;  
     
   case 43   %3d Laplacian
     if (nargin > 3)
       [ A ] = conv_diff(n1, n2, n3);
     elseif (nargin >2)
       [ A ] = conv_diff(n1, n2, n2);
     elseif (nargin >1)
       [ A ] = conv_diff(n1, n1, n1);
     else
       [ A ] = conv_diff(30, 20, 10); 
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n========== Laplacian 3d,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 3d, mode =', mode, ', n =', num2str(n)]
     filename ='lapla3d';
     opert = 1e-2;     
    
   case 44   %2d Laplacian (L-shape)
     if (nargin > 1)
       n1 = max(floor(n1), 50);
       [ A ] =  delsq(numgrid('L', n1));
     else
       [ A ] =  delsq(numgrid('L', 250)); n1=250;
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('\n========== delsq(numgrid(%s,%i)), x1=ones(n,1), n=%i\n','L',n1,n)
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d (L-shape), mode =', mode, ', n =', num2str(n)]
     filename ='lapla2dL';
     opert = 1e-2;  
     
   case 45   %2d Laplacian (H-shape)
     if (nargin > 1)
       n1 = max(floor(n1), 55);
       [ A ] =  delsq(numgrid('H', n1));
     else     
       [ A ] =  delsq(numgrid('H', 200)); n1=200;
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('\n========== delsq(numgrid(%s,%i)), x1=ones(n,1), n=%i\n','H',n1,n)
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d (H-shape), mode =', mode, ', n =', num2str(n)]
     filename ='lapla2H';
     opert = 1e-2;
     
   case 46   %2d Laplacian (A-shape)
     if (nargin > 1)
       n1 = max(floor(n1), 50);
       [ A ] =  delsq(numgrid('A', n1));
     else    
       [ A ] =  delsq(numgrid('A', 150)); n1=150;
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('\n========== delsq(numgrid(%s,%i)), x1=ones(n,1), n=%i\n','A',n1,n)
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d (A-shape), mode =', mode, ', n =', num2str(n)]
     filename ='lapla2A';
     opert = 1e-2;   
   
   case 47   %2d Laplacian (nested dissection ordering of the square).
     if (nargin > 1)
       n1 = max(floor(n1), 40);
       [ A ] =  delsq(numgrid('N', n1));
     else  
       [ A ] =  delsq(numgrid('N', 160)); n1=160;
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('\n========== delsq(numgrid(%s,%i)), x1=ones(n,1), n=%i\n','N',n1,n)
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d (Nested), mode =', mode, ', n =', num2str(n)]
     filename ='lapla2N';
     opert = 1e-2;       
     
   case 48   %2d Laplacian (D-shape ordering of the square).
     if (nargin > 1)
       n1 = max(floor(n1), 40);
       [ A ] =  delsq(numgrid('D', n1));
     else  
       [ A ] =  delsq(numgrid('D', 100)); n1=100;
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('\n========== delsq(numgrid(%s,%i)), x1=ones(n,1), n=%i\n','D',n1,n)
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d (Disc), mode =', mode, ', n =', num2str(n)]
     filename ='lapla2D';
     opert = 1e-2;       
          
   case 49   %2d Laplacian (C-shape ordering of the square).
     if (nargin > 1)
       n1 = max(floor(n1), 40);
       [ A ] =  delsq(numgrid('C', n1));
     else  
       [ A ] =  delsq(numgrid('C', 100)); n1=100;
     end
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('\n========== delsq(numgrid(%s,%i)), x1=ones(n,1), n=%i\n','C',n1,n)
     fprintf(' real SPD Model \n')
     Title = [' Laplacian 2d (C-shape), mode =', mode, ', n =', num2str(n)]
     filename ='lapla2C';
     opert = 1e-2;     
     
     
   %%  
   %--- id=50's, mildly nonsymmetric models from material science 
   %%
   case 50    
     [ A ] = mmread('si5h12.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   si5h12.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real nonsymmetric, si5h12 Model \n')
     Title = [' si5h12, real nonsymmetric, mode =', mode, ', n =', num2str(n)]
     filename ='si5h12';
     opert = 1e-1;         
     
   case 51    
     [ A ] = mmread('GaAsH6.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   GaAsH6.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real nonsymmetric, GaAsH6 Model \n')
     Title = [' GaAsH6, real nonsymmetric, mode =', mode, ', n =', num2str(n)]
     filename ='GaAsH6';
     opert = 1e-1;         
     
   case 52    
     [ A ] = mmread('Ga3As3H12.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Ga3As3H12.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real nonsymmetric, Ga3As3H12 Model \n')
     Title = [' Ga3As3H12, real nonsymmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ga3As3H12';
     opert = 1e-1; 
     
   case 53    
     [ A ] = mmread('Ga10As10H30.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Ga10As10H30.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real nonsymmetric, Ga10As10H30 Model \n')
     Title = [' Ga10As10H30, real nonsymmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Ga10As10H30';
     opert = 1e-1;  
     
   case 54    
     [ A ] = mmread('SiNa.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   SiNa.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real nonsymmetric, SiNa Model \n')
     Title = [' SiNa, real nonsymmetric, mode =', mode, ', n =', num2str(n)]
     filename ='SiNa';
     opert = 1e-1;        

   case 55    
     [ A ] = mmread('Na5.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   Na5.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real nonsymmetric, Na5 Model \n')
     Title = [' Na5, real nonsymmetric, mode =', mode, ', n =', num2str(n)]
     filename ='Na5';
     opert = 1e-1;      
     
     
   %%
   %-- id=60's   (some matrices from the NIST matrix market
   %%
   case 61
     [ A ] = mmread('lshp3466.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'LR';
     fprintf('  \n==========   lshp3466.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric indefinite, lshp3466 Model \n')
     Title = [' lshp3466, real symm indefinite, mode =', mode, ', n =', num2str(n)]
     filename ='lshp3k';
     opert = 1e-2;   
     
   case 62
     [ A ] = mmread('bcsstm24.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   bcsstm24.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real SPD, bcsstm24 Model \n')
     Title = [' bcsstm24, real SPD, mode =', mode, ', n =', num2str(n)]
     filename ='bcsstm24';
     opert = 1e-2;   
   
   case 63
     [ A ] = mmread('bcsstk24.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   bcsstk24.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symm indef, bcsstk24 Model \n')
     Title = [' bcsstk24, real symm indef, mode =', mode, ', n =', num2str(n)]
     filename ='bcsstk24';
     opert = 1e-2;        
     
   case 64 %very bad gap for the lower end spectrum, 1e-16 followed by 1e-13
     [ A ] = mmread('plat1919.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'SR';
     fprintf('  \n==========   plat1919.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symm indef, plat1919 Model \n')
     Title = [' plat1919, real symm indef, mode =', mode, ', n =', num2str(n)]
     filename ='plat1919';
     opert = 1e-2; 
     
     
   case 65
     [ A ] = mmread('cry10000.mtx');
     n = size(A,1);      Matsymm=0;
     x1 = ones(n,1);     mode = 'LR';
     fprintf('  \n==========   cry10000.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real unsymmetric, cry10000 Model \n')
     Title = [' cry10000, real symmetric, mode =', mode, ', n =', num2str(n)]
     filename ='cry10k';
     opert = 1e-2;   
     
     
   case 66  %mode = 'SR' is difficult
     [ A ] = mmread('s3dkt3m2.mtx');
     n = size(A,1);      Matsymm=1;
     x1 = ones(n,1);     mode = 'LR';
     fprintf('  \n==========   s3dkt3m2.mtx,  x1=ones(n,1),    n = %i\n', n)    
     fprintf(' real symmetric, s3dkt3m2  Model \n')
     Title = [' s3dkt3m2, real symmetric, mode =', mode, ',  n =', num2str(n)]
     filename ='s3dkt3m2';
     opert = 1e-1;   

     
   case 1111  %%horrible model, cond=8e+16. fix the model first, man.
     [ A ] = mmread('qh882.mtx');
     fprintf('compute all eigenvalues a + b i in a box of the complex plane \n');
     n = size(A,1);   mode ='SM';   Matsymm=0;
     x1 = ones(n,1);
     Title = [' qh882,   real unsymmetric,   mode =', mode, ',  n =', num2str(n)]
     filename ='qh882';
     opert=1e-8;

   otherwise
     error('no such test matrices')
    
  end
  
  nnzA = nnz(A);
  sizeA= size(A,1);
  fprintf('size(A,1)=%i,   nnz=%i,  norm1 = %e, density=%e\n', ...
	  sizeA, nnzA,  norm(A,1), nnzA/sizeA^2)